﻿using Microsoft.Extensions.Logging;
using RouteGuideContracts.BindingModels;
using RouteGuideContracts.BusinessLogicContracts;
using RouteGuideContracts.ViewModels;
using RouteGuideDataModels.Models;
using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace RouteGuide
{
    public partial class FormTransport : Form
    {
        private readonly ILogger _logger;
        private readonly ITransportLogic _logicT;
        private readonly IRouteLogic _logicR;
        private readonly ITransportTypeLogic _logicTT;
        private List<RouteViewModel>? _listR;
        private List<TransportTypeViewModel>? _listTT;


        public FormTransport(ILogger<FormTransport> logger, ITransportLogic logicT, IRouteLogic logicR, ITransportTypeLogic logicTT)
        {
            InitializeComponent();
            _logger = logger;
            _logicT = logicT;
            _logicR = logicR;
            _logicTT = logicTT;
        }

        public int TransportTypeId { get { return Convert.ToInt32(comboBoxTransportType.SelectedValue); } set { comboBoxTransportType.SelectedValue = value; } }

        public int RouteId { get { return Convert.ToInt32(comboBoxRoute.SelectedValue); } set { comboBoxRoute.SelectedValue = value; } }

        public ITransportTypeModel? TransportTypeModel
        {
            get
            {
                if (_listTT == null)
                {
                    return null;
                }
                foreach (var elem in _listTT)
                {
                    if (elem.Id == TransportTypeId)
                    {
                        return elem;
                    }
                }
                return null;
            }
        }

        public IRouteModel? RouteModel
        {
            get
            {
                if (_listR == null)
                {
                    return null;
                }
                foreach (var elem in _listR)
                {
                    if (elem.Id == RouteId)
                    {
                        return elem;
                    }
                }
                return null;
            }
        }

        private void FormTransport_Load(object sender, EventArgs e)
        {
            _logger.LogInformation("Загрузка транспорта");
            _listTT = _logicTT.ReadList(null);
            _listR = _logicR.ReadList(null);
            if (_listTT != null)
            {
                comboBoxTransportType.DisplayMember = "Name";
                comboBoxTransportType.ValueMember = "Id";
                comboBoxTransportType.DataSource = _listTT;
                comboBoxTransportType.SelectedItem = null;

            }

            if (_listR != null)
            {
                comboBoxRoute.DisplayMember = "Name";
                comboBoxRoute.ValueMember = "Id";
                comboBoxRoute.DataSource = _listR;
                comboBoxRoute.SelectedItem = null;

            }
        }

        private void ButtonSave_Click(object sender, EventArgs e)
        {
            if (string.IsNullOrEmpty(textBoxDriverName.Text))
            {
                MessageBox.Show("Заполните поле Количество", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            if (comboBoxTransportType.SelectedValue == null)
            {
                MessageBox.Show("Выберите драгоценность", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            if (comboBoxRoute.SelectedValue == null)
            {
                MessageBox.Show("Выберите драгоценность", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            _logger.LogInformation("Создание заказа");
            try
            {
                var operationResult = _logicT.Create(new TransportBindingModel
                {
                    TransportTypeId = Convert.ToInt32(comboBoxTransportType.SelectedIndex) + 1,
                    RouteId = Convert.ToInt32(comboBoxRoute.SelectedIndex) + 1,
                });
                if (!operationResult)
                {
                    throw new Exception("Ошибка при создании заказа. Дополнительная информация в логах.");
                }
                MessageBox.Show("Сохранение прошло успешно", "Сообщение", MessageBoxButtons.OK, MessageBoxIcon.Information);
                DialogResult = DialogResult.OK;
                Close();
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка создания заказа");
                MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void ButtonCancel_Click(object sender, EventArgs e)
        {
            DialogResult = DialogResult.Cancel;
            Close();
        }
    }
}
