﻿using RouteGuideContracts.BindingModels;
using RouteGuideContracts.SearchModels;
using RouteGuideContracts.StoragesModels;
using RouteGuideContracts.ViewModels;
using RouteGuideDatabaseImplements.Models;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace RouteGuideDatabaseImplements.Implements
{
    public class TransportStorage : ITransportStorage
    {
        public TransportViewModel? Delete(TransportBindingModel model)
        {
            using var context = new RouteGuideDatabase();

            var element = context.Transports.FirstOrDefault(rec => rec.Id == model.Id);

            if (element != null)
            {
                context.Transports.Remove(element);
                context.SaveChanges();
                return element.GetViewModel;
            }

            return null;
        }

        public TransportViewModel? GetElement(TransportSearchModel model)
        {
            if (string.IsNullOrEmpty(model.DriverName) && !model.Id.HasValue)
            {
                return null;
            }

            using var context = new RouteGuideDatabase();

            return context.Transports.FirstOrDefault(x => (!string.IsNullOrEmpty(model.DriverName) && x.DriverName == model.DriverName) || (model.Id.HasValue && x.Id == model.Id))?.GetViewModel;
        }

        public List<TransportViewModel> GetFilteredList(TransportSearchModel model)
        {
            if (string.IsNullOrEmpty(model.DriverName))
            {
                return new();
            }

            using var context = new RouteGuideDatabase();

            return context.Transports.Where(x => x.DriverName.Contains(model.DriverName)).ToList().Select(x => x.GetViewModel).ToList();
        }

        public List<TransportViewModel> GetFullList()
        {
            using var context = new RouteGuideDatabase();

            return context.Transports.ToList().Select(x => x.GetViewModel).ToList();
        }

        public TransportViewModel? Insert(TransportBindingModel model)
        {
            using var context = new RouteGuideDatabase();

            var newTransport = Transport.Create(model);

            if (newTransport == null)
            {
                return null;
            }

            context.Transports.Add(newTransport);
            context.SaveChanges();

            return newTransport.GetViewModel;
        }

        public TransportViewModel? Update(TransportBindingModel model)
        {
            using var context = new RouteGuideDatabase();

            using var transaction = context.Database.BeginTransaction();

            try
            {
                var transport = context.Transports.FirstOrDefault(rec => rec.Id == model.Id);

                if (transport == null)
                {
                    return null;
                }

                transport.Update(model);
                context.SaveChanges();
                transaction.Commit();

                return transport.GetViewModel;
            }
            catch
            {
                transaction.Rollback();

                throw;
            }
        }
    }
}
